import asyncio

import pytest

from aiormq.tools import LazyCoroutine


@pytest.mark.asyncio
class TestLazyCoroutine:
    async def test_coro(self, event_loop):
        async def foo():
            await asyncio.sleep(0, loop=event_loop)
            return 42

        bar = LazyCoroutine(foo)

        assert await bar == 42

    async def test_future(self, event_loop):
        def foo():
            f = event_loop.create_future()
            event_loop.call_soon(f.set_result, 42)
            return f

        bar = LazyCoroutine(foo)

        assert await bar == 42

    async def test_task(self, event_loop):
        def foo():
            async def inner():
                await asyncio.sleep(0, loop=event_loop)
                return 42

            return event_loop.create_task(inner())

        bar = LazyCoroutine(foo)

        assert await bar == 42



from pamqp import specification as spec

from . import types
from .connection import Connection, connect
from .channel import Channel
from .exceptions import (
    AMQPChannelError,
    AMQPConnectionError,
    AMQPError,
    AMQPException,
    AuthenticationError,
    ChannelAccessRefused,
    ChannelClosed,
    ChannelLockedResource,
    ChannelNotFoundEntity,
    ChannelPreconditionFailed,
    ConnectionChannelError,
    ConnectionClosed,
    ConnectionCommandInvalid,
    ConnectionFrameError,
    ConnectionInternalError,
    ConnectionNotAllowed,
    ConnectionNotImplemented,
    ConnectionResourceError,
    ConnectionSyntaxError,
    ConnectionUnexpectedFrame,
    DeliveryError,
    DuplicateConsumerTag,
    IncompatibleProtocolError,
    InvalidFrameError,
    MethodNotImplemented,
    ProbableAuthenticationError,
    ProtocolSyntaxError,
)

from .version import (
    author_info, package_info, package_license, team_email,
    version_info, __author__, __version__,
)


__all__ = (
    '__author__',
    '__version__',
    'AMQPChannelError',
    'AMQPConnectionError',
    'AMQPError',
    'AMQPException',
    'AuthenticationError',
    'author_info',
    'Channel',
    'ChannelAccessRefused',
    'ChannelClosed',
    'ChannelLockedResource',
    'ChannelNotFoundEntity',
    'ChannelPreconditionFailed',
    'connect',
    'Connection',
    'ConnectionChannelError',
    'ConnectionClosed',
    'ConnectionCommandInvalid',
    'ConnectionFrameError',
    'ConnectionInternalError',
    'ConnectionNotAllowed',
    'ConnectionNotImplemented',
    'ConnectionResourceError',
    'ConnectionSyntaxError',
    'ConnectionUnexpectedFrame',
    'DeliveryError',
    'DuplicateConsumerTag',
    'IncompatibleProtocolError',
    'InvalidFrameError',
    'MethodNotImplemented',
    'package_info',
    'package_license',
    'ProbableAuthenticationError',
    'ProtocolSyntaxError',
    'spec',
    'team_email',
    'types',
    'version_info',
)



import asyncio
from functools import wraps

from yarl import URL


def censor_url(url: URL):
    if url.password is not None:
        return url.with_password('******')
    return url


def shield(func):
    async def awaiter(future):
        return await future

    @wraps(func)
    def wrap(*args, **kwargs):
        return wraps(func)(awaiter)(asyncio.shield(func(*args, **kwargs)))

    return wrap


def _inspect_await_method():
    async def _test():
        pass

    coro = _test()
    method_await = getattr(coro, '__await__', None)
    method_iter = getattr(coro, '__iter__', None)

    for _ in (method_await or method_iter)():
        pass

    return bool(method_await)


HAS_AWAIT_METHOD = _inspect_await_method()


class LazyCoroutine:
    __slots__ = '__func', '__args', '__kwargs', '__instance'

    def __init__(self, func, *args, **kwargs):
        self.__func = func
        self.__args = args
        self.__kwargs = kwargs
        self.__instance = None

    def __repr__(self):
        return "<%s: %s(args: %r, kwargs: %r)>" % (
            self.__class__.__name__, self.__func.__name__,
            self.__args, self.__kwargs
        )

    def __call__(self):
        if self.__instance is None:
            self.__instance = self.__func(*self.__args, **self.__kwargs)

        return self.__instance

    def __iter__(self):
        return (yield from self().__iter__())

    if HAS_AWAIT_METHOD:
        def __await__(self):
            return (yield from self().__await__())
    else:
        def __await__(self):
            return (yield from self().__iter__())
